function SelectedRanks = capcurvemaster(TotalAcc,dT,varargin)

%-------------------------------------------------------------------------
%-------------------------------------------------------------------------
% function capcurvemaster(TotalAcc,dT,...)
% Written by Trevor Yeow
% v3.0 7th May 2022
%-------------------------------------------------------------------------
% BACKGROUND
% This code is written to derive a structure's capacity curve based on
% recorded acceleration response. The basic premise of the methodology is
% to use the discrete wavelet transform approach to decompose the
% acceleration data, select the components which correspond to the
% predominant response mode of the building, double integrate the signals
% to obtain displacement response, then simplify the overall building
% response into an equivalent SDOF system. The general concept is outlined
% in the following reference:
% Kusunoki K, Hinata D, Hattori Y, and Tasai A. (2018). 'A new method for
% evaluating the real-time residual seismic capacity of existing structures
% using accelerometers: structures with multiple degrees of freedom'
%
% The methodology has since been advanced by modifying the method to select
% the signal components to obtain the predominant response. This is based
% on various properties of the decomposed signals, such as peak
% displacement, peak acceleration, slope, and coefficient of determination
% of a linear best-fit line. These are detailed in the following reference:
% Yeow TZ and Kusunoki K. (in-draft). 'Calibrated framework for extracting
% capacity curves of multistory buildings using acceleration data for
% post-earthquake damage evaluations'
%
%-------------------------------------------------------------------------
% IMPORTANT INFORMATION
% This function requires the use of the Signal Processing Toolbox on Matlab
%-------------------------------------------------------------------------
% REQUIRED INPUTS
% TotalAcc   - A matrix of acceleration values. These should be normalized
%              to units of g. Each column represents different floors, and
%              each row corresponds to increasing time. It is assumed that
%              the sampling rate is constant.
% dT         - Timestep of recorded data
%-------------------------------------------------------------------------
% OPTIONAL INPUTS
% Notes: when assigning to call function, input correct variable name as
% string, followed by variable value or string. If these variables are not
% provided, either default values will be used (for Mass and wname) or
% outputs (i.e., plots and summary data files) will not be generated
%
% Mass       - A row of floor masses INCLUDING the ground floor (can be
%              specified as 0, as long as there is an entry provided for
%              the ground floor). Each vector entry corresponds to
%              increasing floor heights. Note that the value of the mass
%              is not important as long as the ratio between masses on
%              different floors is correct.
%              (If not provided, it will be assumed that all floor masses
%              are equal)
% wname      - Wavelet shape for analyses (see wfilters for more info)
%              NOTE: the methodology was only calibrated for db10 or sym10
%                    mother wavelet. It is highly recommended to only use
%                    one of the two.
%              (If not provided, sym10 will be used by default)
% outrankd   - Filename for detailed rank representative acceleration/disp
%              output
% outrankp   - Filename for summary of key rank properties output
% outcombd   - Filename for summary of interstory response
% outscurve  - Filename for skeleton curve output
% PlotFlag   - 1 = plot representative response and skeleton curves
%              (no plots provided for any other number used for flags)
%-------------------------------------------------------------------------
% OUTPUTS
% This function creates four text files which location and name are
% specified by the user (using OUTRANKD, OUTRANKP, OUTCOMBD, OUTSCURVE).
% The files contain the following information:
%     1) Representative acceleration and displacement history for each
%        individual rank (OUTRANKD)
%     2) Key rank properties, such as peak displacement, peak acceleration,
%        slope, and coefficient of determination for each individual rank
%        (OUTRANKP)
%     4) Information of interstory response for each rank (OUTSTORYINFO)
%     3) Representative acceleration and displacement history after
%        combining the resposne of selected ranks. The effective mass ratio
%        and the recomposed ground accelerations are also included.
%        (OUTCOMBD)
%     4) Displacement and acceleration outputs for the skeleton curve (both
%        tentative and final) (OUTSCURVE)
%
% NOTE - accelerations and displacements must act in opposite directions to
% each other. In the text file outputs, there were no corrections to the
% accelerations so these still generally act opposite each other. However,
% in the plot (if the user chooses to display this), the sign of the
% acceleration values were switched.
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------

%% Optional parameters

% Set default valuables/names for optional parameters
Mass = [0, ones(1,length(TotalAcc(1,:))-1)];
PlotFlag = [0, 0, 0];
wname = 'sym10';
outrankd = 0;
outrankp = 0;
outcombd = 0;
outhyst = 0;
outscurve = 0;

% Replace default parameters with user-defined parameters, and create
% output file names if desired by user
for i = 1:length(varargin)/2
    if strcmp(varargin{2*i-1},'Mass')
        Mass = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'PlotFlag')
        PlotFlag = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'wname')
        wname = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'outrankd')
        outrankd = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'outrankp')
        outrankp = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'outcombd')
        outcombd = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'outhyst')
        outhyst = varargin{2*i};
    elseif strcmp(varargin{2*i-1},'outscurve')
        outscurve = varargin{2*i};
    end
end

%% Initial setup

% Identify peak number of ranks to apply
J = wmaxlev(length(TotalAcc(:,1)),wname);

% Determine time vector for analysis output
Time=0:dT:dT*(length(TotalAcc(:,1))-1);

%% STEP 1
%Decompose sensor readings using the DWT method to obtain the jth rank
%total acceleration history for the ith floor

% Decomposing signal to obtain ith floor jth rank total acceleration (xTA)
for i = 1:length(TotalAcc(1,:))
    [c,l] = wavedec(TotalAcc(:,i),J,wname);
    for Rank = 1:J
        xTA(:,Rank+J*(i-1)) = wrcoef('d',c,l,wname,Rank);
    end
end

%% Step 2
% Double intergrate decomposed accelerations and apply linear detrend to
% obtain the jth rank total displacement history for the ith floor

% Double intergrate to obtain total displacement response using trapezium
% rule and apply detrend (xTD - total displacement)
for i = 1:length(xTA(1,:))
    xTD(:,i)=detrend(cumtrapz(cumtrapz(xTA(:,i)))*dT^2*981);
end

%% STEP 3
% Subtract the jth rank ground response from the jth rank upper floor
% response to obtain the relative displacement and acceleration history

% Obtain relative response of ith floor jth rank
% (xRD - relative displacement, xRA - relative acceleration)
xRA = zeros(size(xTA));
xRD = zeros(size(xTA));
for i = 1:J
    xRA(:,i:J:(length(TotalAcc(1,:))-1)*J+i)=xTA(:,i:J:(length(TotalAcc(1,:))-1)*J+i)-xTA(:,i);
    xRD(:,i:J:(length(TotalAcc(1,:))-1)*J+i)=xTD(:,i:J:(length(TotalAcc(1,:))-1)*J+i)-xTD(:,i);
end

%% Step 4
% Apply EQ(1)-(2) to obtain representative acceleration-displacement
% response of each rank, and record (i) peak representative dispalcement,
% (ii) peak representative acceleration, (iii) best-fit linear slope and
% (iv) coefficient of determination R^2

% Side calculation - obtain step corresponding to 5% and 75% of
% energy for ground motion for calculation of MRjeff
Iajnorm = cumtrapz(TotalAcc(:,1).^2)/trapz(TotalAcc(:,1).^2);  %Arias Intensity
T1 = min(find(Iajnorm>0.05));T2 = max(find(Iajnorm<0.75));

for i=1:J
    % Derivation of representative displacement and acceleration of each
    % rank (RA - representative acceleration, RD - representative
    % displacement)using Eq (8) and (9)
    RD(:,i)=sum([xRD(:,i:J:(length(TotalAcc(1,:))-1)*J+i).*Mass],2)./sum(Mass(2:end));         % Application of EQ(1)
    RA(:,i)=sum([xRA(:,i:J:(length(TotalAcc(1,:))-1)*J+i).*Mass],2)./sum(Mass(2:end))+xTA(:,i);    % Application of EQ(2)
    % Derivation of mean effective mass ratio of jth rank using Eq (7)
    MRjeff(i,1)=mean((sum([xRD(T1:T2,i:J:(length(TotalAcc(1,:))-1)*J+i).*Mass],2)).^2./sum([xRD(T1:T2,i:J:(length(TotalAcc(1,:))-1)*J+i).^2.*Mass],2)/sum(Mass(2:end)));
end

% Derive representative relative velocity from representative relative
% displacements
for i = 1:length(RD(:,i))
    if i == 1
        RV(1,:)=zeros(1,length(RD(1,:)));
    else
        RV(i,:)=(RD(i,:)-RD(i-1,:))/dT;
    end
end

% Obtain generic properties of each rank
for i=1:J
    Dj(i,1) = max(abs(RD(:,i)));      % Peak displacement
    Aj(i,1) = max(abs(RA(:,i)));       % Peak acceleration
    PTemp=polyfit(RD(:,i),-RA(:,i),1);
    Kjfit(i,1)=PTemp(1);                    % Slope
    R2Temp=corrcoef(RD(:,i),-RA(:,i));
    Rj2(i,1)=R2Temp(1,2);                % Coefficient of determination R^2
    CEkj(i,1) = trapz(RV(:,i).^2./mean(MRjeff(i,1))*0.5)*dT;  % Cumulative kinetic energy
end

%% Step 5
% Select the rank with the largest cumulative kinetic energy that
% has an acceleration ratio greater than 0.15 as the initial rank

% Set displacement of ranks which do not satisfy ARL to 0 to exclude from
% contention of main rank
MainRank = find(CEkj.*(Aj>0.25*max(Aj))==max(CEkj.*(Aj>0.25*max(Aj))));

% Obtain highest rank
HighRank = max(find(Kjfit.*Aj/(Kjfit(MainRank).*Aj(MainRank))>0.01));

% Obtain lowest rank
LowRank = min(find((Dj./Dj(MainRank)>0.05).*(MRjeff./MRjeff(MainRank)>0.65)==1));

% Obtain selected ranks
SelectedRanks = LowRank:HighRank;

%% STEP 7
% Superimpose building response of selected ranks together, and obtain the
% tentative and final skeleton curve using EQ(3)-(5)

% Superimposing response of ith floor for selected ranks
for i = 1:length(Mass)
    TotalFloorDisp(:,i) = sum(xTD(:,(i-1)*J+min(SelectedRanks):(i-1)*J+max(SelectedRanks)),2);
    TotalFloorAcc(:,i) = sum(xTA(:,(i-1)*J+min(SelectedRanks):(i-1)*J+max(SelectedRanks)),2);
end
RelativeFloorDisp=TotalFloorDisp-TotalFloorDisp(:,1);
RelativeFloorAcc=TotalFloorAcc-TotalFloorAcc(:,1);

% Calculation of effective mass ratio using EQ(3)
EMR = sum([RelativeFloorDisp.*Mass],2).^2./(sum([RelativeFloorDisp.^2.*Mass],2).*sum(Mass(2:end)));

% Representative displacement and acceleration based on superimposed
% response using Eq(1)-(2)
RDcombinedTen=sum([RelativeFloorDisp.*Mass],2)./sum(Mass(2:end));                     % Application of EQ(1)
RAcombinedTen=sum([RelativeFloorAcc.*Mass],2)./sum(Mass(2:end))+TotalFloorAcc(:,1);   % Application of EQ(2)

% Representative displacement and acceleration based on superimposed
% response using Eq(1)-(2)
RDcombined=sum([RelativeFloorDisp.^2.*Mass],2)./sum([RelativeFloorDisp.*Mass],2);                     % Application of EQ(1)
RAcombined=sum([RelativeFloorDisp.^2.*Mass],2)./sum([RelativeFloorDisp.*Mass],2).^2.*sum([RelativeFloorAcc.*Mass],2)+TotalFloorAcc(:,1);   % Application of EQ(2)

% Extract out corrected hysteresis
counter = 0;
for i = 1:length(EMR)
    if EMR(i)>0.60
        counter=counter+1;
        Hyst(counter,:)=[RDcombined(i),-RAcombined(i)];
    end
end

% Extracting out the tentative capacity curve
SkeletonCurve = [0 0 1 0];
counter = 1;
for i = 1:length(RDcombined)
    if RDcombined(i)>max(SkeletonCurve(:,1)) || RDcombined(i)<min(SkeletonCurve(:,1))
        if EMR(i)>0.60
            counter = counter+1;
            SkeletonCurve(counter,:)=[RDcombined(i) -RAcombined(i) EMR(i) TotalFloorAcc(i,1)];
        end
    end
end

% Rearranging to obtain displacements in ascending order
SkeletonCurve = sortrows(SkeletonCurve,1);

%% Create plots

% Plot individual rank representative response
if PlotFlag(1) == 1
    xlimval = ceil(max(max(abs(RD(:,SelectedRanks))))/5)*5;
    ylimval = ceil(max(max(abs(RA(:,1:max(SelectedRanks)))))/0.5)*0.5;
    n1 = ceil(sqrt(J));n2 = ceil(J/n1);
    s = get(0, 'ScreenSize');
    H = figure(1);
    set(H,'Position', [0 0 s(3) s(4)]);
    Plot1 = tiledlayout(n2,n1,'TileSpacing','Compact');
    for i = 1:J
        nexttile;
        plot(RD(:,i),-RA(:,i));
        xlim([-xlimval xlimval]);ylim([-ylimval ylimval]);
        text(xlimval*-0.9,ylimval*0.7,['Rank ',num2str(i)],'fontsize',18);
        grid;
        set(gca,'fontsize',14);
    end
    xlabel(Plot1,'Representative Displacement (cm)','fontsize',18);
    ylabel(Plot1,'Representative Acceleration (g)','fontsize',18);
end

% Plot tentative hysteretic response
if PlotFlag(2) == 1
    xlimval = ceil(max(abs(RDcombinedTen))/5)*5;
    ylimval = ceil(max(abs(RAcombinedTen))/0.5)*0.5;
    figure(2)
    plot(RDcombinedTen,-RAcombinedTen,'r','linewidth',2)
    grid
    xlabel('Representative displacement (cm)')
    ylabel('Representative acceleration (g)')
    xlim([-xlimval xlimval]);ylim([-ylimval ylimval]);
    set(gca,'fontsize',16)
end

% Plot backbone curve
if PlotFlag(3) == 1
    xlimval = ceil(max(abs(SkeletonCurve(:,1)))/5)*5
    ylimval = ceil(max(abs(SkeletonCurve(:,2)))/0.5)*0.5
    figure(3)
    plot(SkeletonCurve(:,1),SkeletonCurve(:,2),'r','linewidth',3)
    grid
    xlabel('Representative displacement (cm)')
    ylabel('Representative acceleration (g)')
    set(gca,'fontsize',16)
    xlim([-xlimval xlimval]);ylim([-ylimval ylimval]);
end

%% Write files

% Export rank response data
if outrankd ~= 0
    for i=1:J
        FMHeading{i} = ['Rank',num2str(i),'Disp(cm) '];
        FMHeading{J+i} = ['Rank',num2str(i),'Acc(g) '];
    end
    writecell(FMHeading, outrankd,'delimiter',',');
    writematrix([RD -RA],outrankd,'writemode','append');
end

% Export rank properties
if outrankp ~= 0
    writecell({'Rank','PeakDisp(cm)','PeakAcc(g)','MeanEffMass','Slope(g/cm)','CumulativeKEnergy(cm^2/s)'}, outrankp,'delimiter',',');
    writematrix([[1:J]',Dj,Aj,MRjeff,Kjfit,CEkj],outrankp,'writemode','append');
end

% Export representative displacement and acceleration results for combined
% ranks
if outcombd ~= 0
    for i = 1:length(Mass)
            Outputheader{i} = ['RelDisp',num2str(i),'F(cm)'];
            Outputheader{i+length(Mass)} = ['RelAcc',num2str(i),'F(g)'];
            Outputheader{i+2*length(Mass)} = ['TotalAcc',num2str(i),'F(g)'];
    end

    writecell(Outputheader, outcombd,'delimiter',',');
    writematrix([RelativeFloorDisp,RelativeFloorAcc,TotalFloorAcc],outcombd,'writemode','append');
end

% Exported hysteresis curve
if outhyst ~= 0
    writecell({'TentRepDisp(cm)','TentRepAcc(g)'}, outhyst,'delimiter',',');
    writematrix([RDcombinedTen,-RAcombinedTen],outhyst,'writemode','append');
end

% Export skeleton curve
if outscurve ~= 0
    writecell({'BCRepDisp(cm)','BCRepAcc(g)'}, outscurve,'delimiter',',');
    writematrix(SkeletonCurve(:,1:2),outscurve,'writemode','append');
end

